@@ -1,5 +1,5 @@
 #__all__ = ['guide', 'model', 'stopping_criteria', 'calculate_posteriors',
-#           'interface', 'example_loading', 'plotting', 'model_selection']
+#           'interface', 'example_loading', 'plotting', 'model_selection', 'BoundedPareto']
 
 from mobster.guide_mobster import guide
 from mobster.model_mobster import model
@@ -7,3 +7,4 @@ from mobster.interface_mobster import fit_mobster
 from mobster.example_loading import *
 from mobster.plotting_mobster import *
 from mobster.stopping_criteria import *
+from mobster.BoundedPareto import *
