@@ -4,5 +4,5 @@ import torch
 def load_example_data(directory = "./"):
     flh = open(directory + "example.pkl", "rb")
     inp = pickle.load(flh)
-    inp = {k: (torch.tensor(v) - 0.001) for k, v in zip(inp.keys(), inp.values())}
+    inp = {k: v.float().round() for k, v in zip(inp.keys(), inp.values())}
     return inp
\ No newline at end of file
